==> takeaction-0.1.tgz: takeaction/takeaction.py <==
# takeaction: content as actions
# Copyright (C) 2007 Plone Solutions AS
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
from zope.interface import implements

from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from AccessControl.Permissions import view
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager

from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFPlone.interfaces.Translatable import ITranslatable

from interfaces import ITakeActionTool

class TakeActionTool(UniqueObject, SimpleItem, PropertyManager, 
                     ActionProviderBase):
    """TakeAction tool; designate content as actions"""
    
    implements(ITakeActionTool)
    
    __implements__ = ActionProviderBase.__implements__
    
    id = "portal_takeaction"
    meta_type = 'TakeAction content-as-actions tool'
    content_paths = ()
    category = 'take_actions'
    
    _properties = (
        dict(id='content_paths', type='lines', mode='w'),
        dict(id='category', type='string', mode='w'),
    )
    manage_options = PropertyManager.manage_options + SimpleItem.manage_options
    security = ClassSecurityInfo()
    
    security.declarePrivate('listActions')
    def listActions(self, info=None, object=None):
        portal = getToolByName(self, 'portal_url').getPortalObject()
        actions = []
        for i, path in enumerate(self.content_paths):
            # No point in using the catalog here, as we are trying to find
            # the correct translation
            content = portal.unrestrictedTraverse(path, None)
            if content is None:
                continue
            if ITranslatable.isImplementedBy(content):
                content = content.getTranslation()
            if not getSecurityManager().checkPermission(view, content):
                continue
            action = ActionInformation(
                'takeaction%02d' % i,
                content.Title(),
                content.Description(),
                self.category,
                action='string:' + content.absolute_url())
            actions.append(action)
        return actions
            
InitializeClass(TakeActionTool)

==> takeaction-0.3.tgz: takeaction/takeaction.py <==
# takeaction: content as actions
# Copyright (C) 2007 Plone Solutions AS
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
from zope.interface import implements

from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from AccessControl.Permissions import view
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager

from Products.CMFCore.ActionInformation import ActionInformation
from Products.CMFCore.ActionProviderBase import ActionProviderBase
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.CMFPlone.interfaces.Translatable import ITranslatable

from interfaces import ITakeActionTool

class TakeActionTool(UniqueObject, SimpleItem, PropertyManager, 
                     ActionProviderBase):
    """TakeAction tool; designate content as actions"""
    
    implements(ITakeActionTool)
    
    __implements__ = ActionProviderBase.__implements__
    
    id = "portal_takeaction"
    meta_type = 'TakeAction content-as-actions tool'
    content_paths = ()
    category = 'take_actions'
    
    _properties = (
        dict(id='content_paths', type='lines', mode='w'),
        dict(id='category', type='string', mode='w'),
    )
    manage_options = PropertyManager.manage_options + SimpleItem.manage_options
    security = ClassSecurityInfo()
    
    security.declarePrivate('listActions')
    def listActions(self, info=None, object=None):
        portal = getToolByName(self, 'portal_url').getPortalObject()
        actions = []
        ids = set()
        for i, path in enumerate(self.content_paths):
            # No point in using the catalog here, as we are trying to find
            # the correct translation
            content = portal.unrestrictedTraverse(path, None)
            if content is None:
                continue
            content_id = content.getId()
            if ITranslatable.isImplementedBy(content):
                translation = content.getTranslation()
                if translation is None:
                    # Try falling back to a language-neutral version
                    translation = content.getTranslation('')
                if translation is None:
                    continue
                content = translation
            if not getSecurityManager().checkPermission(view, content):
                continue
            # Ensure that we use a unique id for our action
            id = 'takeaction-%s' % content_id
            idcount = 1
            while id in ids:
                idcount += 1
                id = 'takeaction-%s-%d' % (content_id, idcount)
            ids.add(id)
            action = ActionInformation(
                id,
                content.Title(),
                content.Description(),
                self.category,
                action='string:' + content.absolute_url())
            actions.append(action)
        return actions
            
InitializeClass(TakeActionTool)
